local math = math

local P = {}

P.TCPOINT = {
    POINT_A = 1,
    POINT_B = 2,
    POINT_C = 3,
    SIDE_AB = 4,
    SIDE_BC = 5,
    SIDE_CA = 6,
    INSIDE = 7,
    OUTSIDE = 8,
}

---由A-->B-->C-->A 按逆时针方向转形成的三角形面积
--@param #point_table pointA
--@param #point_table pointB
--@param #point_table pointC
--@return #float 三角形面积
function P.triangleArea(pointA, pointB, pointC)
    return math.abs((pointA.x*pointB.y+pointB.x*pointC.y+pointC.x*pointA.y-pointA.x*pointC.y-pointB.x*pointA.y-pointC.x*pointB.y))/2;
end

---点point位于由A-->B-->C-->A 按逆时针方向转形成的三角形的位置
--@param #point_table tpA
--@param #point_table tpB
--@param #point_table tpC
--@param #point_table point
--@return #P.TCPOINT 位置类型
function P.triangleContainsPoint(tpA, tpB, tpC, point)
    if tpA.x == point.x and tpA.y == point.y then
    	return P.TCPOINT.POINT_A
    elseif tpB.x == point.x and tpB.y == point.y then
        return P.TCPOINT.POINT_B
    elseif tpC.x == point.x and tpC.y == point.y then
        return P.TCPOINT.POINT_C
    end
     
    local areaABC = P.triangleArea(tpA,tpB,tpC)
    local areaABX = P.triangleArea(tpA,tpB,point)
    local areaBCX = P.triangleArea(tpB,tpC,point)
    local areaCAX = P.triangleArea(tpC,tpA,point)
    
    if areaABX == 0 then
    	return P.TCPOINT.SIDE_AB
    elseif areaBCX == 0 then
        return P.TCPOINT.SIDE_BC
    elseif areaCAX == 0 then
        return P.TCPOINT.SIDE_CA
    end
    
    if areaABC == (areaABX + areaBCX + areaCAX) then
    	return P.TCPOINT.INSIDE
    end
    
    return P.TCPOINT.OUTSIDE
end

---原点Point::ZERO为起始点，终点为(moveByX,moveByY), 终点同时为Control Point2
---原点(A)与终点(B)的连线为线段(AB),中心点为(C)
---Control Point1是线段(AB)的中垂线(CL)上的点（这个点是中垂线(CL)上点(C)到(AB长度×（AB与横轴角度(0~90)/90)的点）
--@param moveByX 相对原点X轴偏移量
--@param moveByY 相对原点Y轴偏移量
--@return 二次方贝塞尔曲线中的Control Point1
function P.getBezierControlPoint1(moveByX, moveByY)
    local interval = math.sqrt(moveByX*moveByX + moveByY*moveByY)
    
    local origin = cc.p(0, 0)
    local endPoint = cc.p(moveByX, moveByY)
    
    local center = cc.p((origin.x + endPoint.x)*0.5, (origin.y + endPoint.y)*0.5)--中点坐标
    local k1 = 0
    if (endPoint.x - origin.x) ~= 0 then
        k1 = (endPoint.y - origin.y)/(endPoint.x - origin.x)--origin到end直线的斜率
    end
    local k2 = 0
    if k1 ~= 0 then
        k2 = -1/k1--中垂线的斜率
    end
    local ctrPointX = center.x
    local ctrPointY = center.y
    
    local centerLineLen = 0
    local maxCLineLen = 0
    local rate = 1
    if endPoint.y == 0 then
        maxCLineLen = interval*rate
    else
        local tanVal = math.tan(moveByX/moveByY)
        local pi_angle = math.atan(tanVal)
        --log("pi_angle=%f",pi_angle)
        --0度时,interval为最大值,角度增大一度，interval约小1/90
        maxCLineLen= interval*(rate*((3.14*.5 - math.abs(pi_angle))/(3.14*.5)))
    end
    
    while centerLineLen < maxCLineLen do
        if (moveByX > 0 and moveByY < 0) or (moveByX < 0 and moveByY > 0) then--2，4区间
            ctrPointX = ctrPointX + 1
        else
            ctrPointX = ctrPointX - 1
        end
        --y-(y1+y2)/2=k2*[x-（x1+x2)/2]
        ctrPointY = k2*(ctrPointX - (origin.x + endPoint.x) * 0.5) + (origin.y + endPoint.y) * 0.5
        
        centerLineLen = math.sqrt(math.abs(ctrPointX - center.x)*math.abs(ctrPointX - center.x) + math.abs(ctrPointY - center.y)*math.abs(ctrPointY - center.y))
    end
    
    --cclog("getBezierControlPoint1(%f,%f)",ctrPointX,ctrPointY)
    
    return cc.p(ctrPointX, ctrPointY)
end

---
--@param    originPoint 起始点
--@param    ctrPoint    控制点1
--@param    endPoint    终点&控制点2
--@param    time          时间点
--@return   二次方贝塞尔曲线运行时时间点time的坐标
function P.getPropBezierNextPoint(originPoint, ctrPoint, endPoint, time)
    local nextPoint = cc.p(0,0)
    
    local st = 1 - time
    --二次方贝塞尔曲线公式
    --B(t) = (1-t)(1-t)P0 + 2t(1-t)P1 + (t*t)P2
    nextPoint.x = math.pow(st, 2) * originPoint.x + 2 * time * st * ctrPoint.x + math.pow(time, 2) * endPoint.x
    nextPoint.y = math.pow(st, 2) * originPoint.y + 2 * time * st * ctrPoint.y + math.pow(time, 2) * endPoint.y
    
    return nextPoint
end

---圆心射线与圆相交点坐标
--@param centerPoint 圆心坐标
--@param radius 圆半径
--@param outCirclePoint 圆外一点
--@return 相交点坐标
function P.getIntersectPoint(centerPoint, radius, outCirclePoint)
    local distance = P.calDistance(centerPoint,outCirclePoint)
    local k = radius/distance
    local x = (outCirclePoint.x - centerPoint.x)*k + centerPoint.x
    local y = (outCirclePoint.y - centerPoint.y)*k + centerPoint.y
    
    return cc.p(x, y)
end

function P.calDistance(source, target)
    if source == nil  or target == nil then
        return 0
    end
    local len1 = math.pow(source.x-target.x,2)
    local len2 = math.pow(source.y-target.y,2)
    local dis = math.sqrt(len1+len2)
    return dis
end

return P