local Enum = require("common.Enum")
local LKConfig = require("conf.LKConfig")

local GamePool = require("model.GamePool")

local LayerStage = require("manager.LayerStage")

local MainPopLayer = require("gameUI.layer.MainPopLayer")
local SinglePopLayer = require("gameUI.layer.SinglePopLayer")
local LoadingLayer = require("gameUI.layer.LoadingLayer")
local MultipChoiceLayer = require("gameUI.layer.MultipChoiceLayer")
local NoticeLayer = require("gameUI.layer.NoticeLayer")

local error = error
local cclog = cclog
local cc = cc
local ipairs = ipairs
local table = table
local math = math

local P = {}
local TRAN_DURATION = 0.3
local SCALE_FULL = 1
local SCALE_MIN = 0.01


local tl = cc.Layer:create()
local LAYER_STATUS = {
    NOT_EXIST = -1,
    SHOWING = 0,
    BEING_SHOW = 1,
    HIDING = 2
}

local layers = {}

local function show(layer)
    layer:stopAllActions()
    layer:setScale(SCALE_MIN)

    if layer:getTag() == Enum.LAYER_TYPE.MAIN_POP then
        lk.LakooManager:getInstance():setEnableDrawLKLayerInAction(false)
    end
   
    local targetPos = layer:getTargetPosition(layer:getTag())
    local moveAct = cc.MoveTo:create(TRAN_DURATION, targetPos)
    local scaleAct = cc.ScaleTo:create(TRAN_DURATION, SCALE_FULL)
    local showAct = cc.Spawn:create(moveAct,scaleAct)
    showAct:setTag(LAYER_STATUS.SHOWING)
    
    local callBackAct = cc.CallFunc:create(function()
        lk.LakooManager:getInstance():setEnableDrawLKLayerInAction(true)
    end)
    local seqAct = cc.Sequence:create(showAct,callBackAct)
    
    layer:runAction(seqAct)
    return
end

local function hide(layer, removeFunc)
    layer:stopAllActions()
    local pos = layer:getTargetPosition(layer:getTag())
    local moveAct = cc.MoveTo:create(TRAN_DURATION, pos)
    local scaleAct = cc.ScaleTo:create(TRAN_DURATION, SCALE_MIN)
    local showAct = cc.Spawn:create(moveAct,scaleAct)
    local removeAct = cc.CallFunc:create(removeFunc)
    local seqAct = cc.Sequence:create(showAct,removeAct)

    layer:runAction(seqAct)

end

function P.closeLayerWithType(_type)
    local layer = layers[_type]
    if layer == nil then
        return
    end
    hide(layer, function()
        layers[_type] = nil
        layer:removeFromParent(true)
    end)
end


local LAYER_SWITCH = {
    [Enum.LAYER_TYPE.MAIN_POP] = function() 
        return MainPopLayer:create(P.closeLayerWithType, Enum.LAYER_TYPE.MAIN_POP)
    end,
    [Enum.LAYER_TYPE.MAIN_POP_SOUL] = function() 
        return MainPopLayer:create(P.closeLayerWithType, Enum.LAYER_TYPE.MAIN_POP_SOUL)
    end,
    [Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_WEAPON] = function() 
        return SinglePopLayer:create(P.closeLayerWithType, Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_WEAPON)
    end,
    [Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ARMOUR] = function() 
        return SinglePopLayer:create(P.closeLayerWithType, Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ARMOUR)
    end,
    [Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ADORN_DATA] = function() 
        return SinglePopLayer:create(P.closeLayerWithType, Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ADORN_DATA)
    end,
    [Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP] = function() 
        return SinglePopLayer:create(P.closeLayerWithType, Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP)
    end,
    [Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP] = function() 
        return SinglePopLayer:create(P.closeLayerWithType, Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP)
    end,
    [Enum.LAYER_TYPE.LOADING] = function()
        return LoadingLayer:create()
    end,
    [Enum.LAYER_TYPE.MULTIP_CHOICE] = function()
        return MultipChoiceLayer:create()
    end,
    [Enum.LAYER_TYPE.NOTICE] = function()
        return NoticeLayer:create(P.closeLayerWithType)
    end,
}


function P.showLayerWithType(_type)
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(_type)
    if layer ~= nil then
    	return
    end
    local func = LAYER_SWITCH[_type]
    if func == nil then
        return nil
    else
        layer = func()
    end

    if layer == nil then
        return
    end

    layer:setTag(_type)
    local point = layer:getTargetPosition()
    layer:ignoreAnchorPointForPosition(false)
    layer:setAnchorPoint(cc.p(.5, .5))
    layer:setPosition(point)
    
    scene:addChild(layer)
    layers[_type] = layer
    show(layer)
    
    return layer
end

local function update()
    if LayerStage.getShowLayerType() ~= Enum.LAYER_TYPE.NONE then
    	P.showLayerWithType(LayerStage.getShowLayerType())
    	LayerStage.setShowLayerType(Enum.LAYER_TYPE.NONE)
    end
    
    if LayerStage.getHideLayerType() ~= Enum.LAYER_TYPE.NONE then
    	P.closeLayerWithType(LayerStage.getHideLayerType())
    	LayerStage.setHideLayerType(Enum.LAYER_TYPE.NONE)
    end
end

function P.init()
    local scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)

    local function onNodeEvent(event)
        if "exit" == event then
            --cc.Director:getInstance():getScheduler():unscheduleScriptEntry(scheduleID)
        end
    end
--    local node = cc.Node:create();
--    node:registerScriptHandler(onNodeEvent)
--    
--    if GamePool.getCurrentScene() ~= nil then
--    	GamePool.getCurrentScene():addChild(node)
--    end
end
return P